@@ -3,7 +3,7 @@ package YAHC;
 use strict;
 use warnings;
 
-our $VERSION = '0.010';
+our $VERSION = '0.012';
 
 use EV;
 use Time::HiRes;
@@ -132,11 +132,11 @@ sub request {
 
     my $pool_args = $self->{pool_args};
     $request->{_target} = _wrap_target_selection($request->{host}) if $request->{host};
-    do { $request->{$_} ||= $pool_args->{$_} if $pool_args->{$_} } foreach (qw/host port scheme/);
-    die "YAHC: host must be defined\n" unless $request->{host};
-
+    do { $request->{$_} ||= $pool_args->{$_} if $pool_args->{$_} } foreach (qw/host port scheme request_timeout
+                                                                               connect_timeout drain_timeout/);
     $request->{scheme} //= 'http';
     die "YAHC: only support scheme http\n" unless $request->{scheme} eq 'http';
+    die "YAHC: host must be defined\n" unless $request->{host};
 
     my %callbacks;
     foreach (@{ CALLBACKS() }) {
@@ -170,7 +170,8 @@ sub drop {
     _register_in_timeline($conn, "dropping connection from pool") if $conn->{keep_timeline};
     $self->_set_completed_state($conn_id) unless $conn->{state} == YAHC::State::COMPLETED();
 
-    return delete $self->{connections}{$conn_id};
+    delete $self->{connections}{$conn_id};
+    return $conn;
 }
 
 sub run         { shift->_run(0, @_)            }
@@ -581,12 +582,12 @@ sub _build_socket_and_connect {
 
 sub _get_next_target {
     my $conn = shift;
-    my ($host, $ip, $port) = $conn->{request}{_target}->();
+    my ($host, $ip, $port) = $conn->{request}{_target}->($conn);
 
     # TODO STATE_RESOLVE_DNS
-    ($host, $port) = ($1, $2) if $host =~ m/^(.+):([0-9]+)$/o;
-    $ip = $host if $host =~ m/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/o;
-    $ip = inet_ntoa(gethostbyname($host) or die "Failed to resolve '$host': $!\n") unless $ip;
+    ($host, $port) = ($1, $2) if !$port && $host =~ m/^(.+):([0-9]+)$/o;
+    $ip = $host if !$ip && $host =~ m/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/o;
+    $ip ||= inet_ntoa(gethostbyname($host) or die "Failed to resolve '$host': $!\n");
     $port ||= $conn->{request}{port} || HTTP_PORT;
 
     return @{ $conn->{selected_target} = [ $host, $ip, $port ] };
@@ -717,7 +718,7 @@ sub _register_error {
     my $strerror = sprintf("$format", @arguments);
     $strerror =~ s/\s+$//g;
     _register_in_timeline($conn, "error=$error ($strerror)") if $conn->{debug};
-    push @{ $conn->{errors} ||= [] }, [ $error, $strerror, yahc_conn_target($conn) // 'no_target_yet', Time::HiRes::time ];
+    push @{ $conn->{errors} ||= [] }, [ $error, $strerror, [ @{ $conn->{selected_target} } ], Time::HiRes::time ];
 }
 
 sub _assert_state {
